@@ -1466,7 +1466,14 @@ parse_type_user_defined(
     }
 
     if (give_error && (did_emsg == did_emsg_before))
+    {
+	char_u	*p = skip_type(*arg, FALSE);
+	char	cc = *p;
+
+	*p = NUL;
 	semsg(_(e_type_not_recognized_str), *arg);
+	*p = cc;
+    }
 
     return NULL;
 }
@@ -1894,14 +1901,12 @@ type_name_list_or_dict(char *name, type_T *type, char **tofree)
 
     size_t len = STRLEN(name) + STRLEN(member_name) + 3;
     *tofree = alloc(len);
-    if (*tofree != NULL)
-    {
-	vim_snprintf(*tofree, len, "%s<%s>", name, member_name);
-	vim_free(member_free);
-	return *tofree;
-    }
+    if (*tofree == NULL)
+	return name;
 
-    return name;
+    vim_snprintf(*tofree, len, "%s<%s>", name, member_name);
+    vim_free(member_free);
+    return *tofree;
 }
 
 /*
@@ -1924,17 +1929,15 @@ type_name_class_or_obj(char *name, type_T *type, char **tofree)
 
     size_t len = STRLEN(name) + STRLEN(class_name) + 3;
     *tofree = alloc(len);
-    if (*tofree != NULL)
-    {
-	vim_snprintf(*tofree, len, "%s<%s>", name, class_name);
-	return *tofree;
-    }
+    if (*tofree == NULL)
+	return name;
 
-    return name;
+    vim_snprintf(*tofree, len, "%s<%s>", name, class_name);
+    return *tofree;
 }
 
 /*
- * Return the type name of a functio.
+ * Return the type name of a function.
  * The result may be in allocated memory, in which case "tofree" is set.
  */
     static char *
@@ -2095,10 +2098,12 @@ check_typval_is_value(typval_T *tv)
 	case VAR_CLASS:
 	    {
 		class_T *cl = tv->vval.v_class;
-		if (IS_ENUM(cl))
-		    semsg(_(e_using_enum_as_value_str), cl->class_name);
+		char_u *class_name = (cl == NULL) ? (char_u *)""
+							: cl->class_name;
+		if (cl != NULL && IS_ENUM(cl))
+		    semsg(_(e_using_enum_as_value_str), class_name);
 		else
-		    semsg(_(e_using_class_as_value_str), cl->class_name);
+		    semsg(_(e_using_class_as_value_str), class_name);
 	    }
 	    return FAIL;
 
